package com.install4j.api.beans;

import com.install4j.runtime.installer.InstallerVariables;

import java.io.File;
import java.util.ArrayList;
import java.util.List;

/**
 * Common base class for all abstract super-classes for beans that are handled by install4j.
 * It is not recommended to extend this class directly. Concrete super-classes exist for
 * <ul>
 * <li><a href="../screens/package-summary.html">screens</a></li>
 * <li><a href="../actions/package-summary.html">actions</a></li>
 * <li><a href="../formcomponents/package-summary.html">form components</a></li>
 * </ul>
 * <p>This class provides common utility methods fir all bean types.
 * @author ej-technologies GmbH
 */
public abstract class AbstractBean implements Bean {

    /**
     * Replace all installer variables and localization keys in a string.
     * <p>Note: compiler variables are replaced at compile time.
     * @param value the original string
     * @return the string with all variables replaced.
     */
    protected static String replaceVariables(String value) {
        return InstallerVariables.replaceVariables(value);
    }

    /**
     * Replace all installer variables and localization keys in a string that should be used in a regular expression.
     * <p>Note: compiler variables are replaced at compile time.
     * @param value the original string
     * @param replacementMode the replacement mode
     * @return the string with all variables replaced.
     */
    protected static String replaceVariables(String value, ReplacementMode replacementMode) {
        return InstallerVariables.replaceVariables(value, replacementMode);
    }
    /**
     * Replace all installer variables and localization keys in a file name.
     * <p>Note: compiler variables are replaced at compile time.
     * @param file the original file
     * @return the file with all variables replaced in its name.
     */
    protected static File replaceVariables(File file) {
        return InstallerVariables.replaceVariables(file);
    }

    /**
     * Replace all installer variables and localization keys in a string array.
     * For array elements that consist of an installer variable that in turn contains
     * an array value, the elements of that array value will be inserted into
     * the returned array. For this reason the returned array may have more elements than the
     * original array. For example, if the array passed in as an argument has the elements:
     * <pre>
     *
     *   [0] = "One"
     *   [1] = "${installer:myVariable}"
     *   [2] = "Three"
     *
     * </pre>
     * and the variable <tt>myVariable</tt> has a String array value with the elements
     * <pre>
     *
     *   [0] = "Blue"
     *   [1] = "Green"
     *
     * </pre>
     * then the returned array will be
     * <pre>
     *
     *   [0] = "One"
     *   [1] = "Blue"
     *   [2] = "Green"
     *   [3] = "Three"
     *
     * </pre>
     *<p>
     * If the array in an installer variable is not of type <tt>String[]<tt>,
     * each element will be converted to a string by calling <tt>toString()</tt> on it.
     * </p>
     * <p>Note: compiler variables are replaced at compile time.
     * @param values the original array
     * @return the array with all variables replaced.
     */
    protected static String[] replaceVariables(String[] values) {
        if (values == null) {
            return null;
        } else {

            List expandedArguments = new ArrayList();
            String prefix = "${" + InstallerVariables.INSTALLER_PREFIX;
            for (int i = 0; i < values.length; i++) {
                String argument = values[i];
                if (argument.startsWith(prefix) && argument.indexOf('}') == argument.length() - 1) {
                    String variableName = argument.substring(prefix.length(), argument.length() - 1);
                    Object variableValue = InstallerVariables.getVariable(variableName);
                    if (variableValue instanceof Object[]) {
                        Object[] variableValues = (Object[])variableValue;
                        for (int j = 0; j < variableValues.length; j++) {
                            expandedArguments.add(replaceVariables(variableValues[j].toString()));
                        }
                    } else {
                        expandedArguments.add(replaceVariables(argument));
                    }
                } else {
                    expandedArguments.add(replaceVariables(argument));
                }
            }

            return (String[])expandedArguments.toArray(new String[expandedArguments.size()]);
        }
    }
}
