package com.install4j.api.context;

import com.install4j.runtime.installer.InstallerConstants;

import java.util.HashMap;
import java.util.Map;

/**
 * Enumeration class that represents the different ways how install4j can handle the uninstallation of a file.
 * @author ej-technologies GmbH
 * @see FileOptions
 */
public class UninstallMode {

    private static Map intValueToUninstallMode = new HashMap();

    /**
     * Indicates that if the destination file was created by the installer it will be deleted for uninstallation.
     * This is the default setting.
     */
    public static final UninstallMode IF_CREATED = new UninstallMode("If created", InstallerConstants.UNINSTALL_IF_CREATED);

    /**
     * Indicates that the destination file won't be deleted for uninstallation.
     */
    public static final UninstallMode NEVER = new UninstallMode("Never", InstallerConstants.UNINSTALL_NEVER);

    /**
     * Indicates that the uninstaller will always delete this file. Handle with care.
     */
    public static final UninstallMode ALWAYS = new UninstallMode("Always", InstallerConstants.UNINSTALL_ALWAYS);

    /**
     * Indicates that the destination file won't be deleted when the uninstaller runs as part of an update.
     * Otherwise it behaves like <tt>IF_CREATED</tt>.
     */
    public static final UninstallMode IF_CREATED_BUT_NOT_FOR_UPDATE = new UninstallMode("If created, but not for update", InstallerConstants.UNINSTALL_IF_CREATED_BUT_NOT_FOR_UPDATE);

    /**
     * Indicates that the destination file won't be deleted when the uninstaller runs as part of an update.
     * Otherwise it behaves like <tt>ALWAYS</tt>.
     */
    public static final UninstallMode ALWAYS_BUT_NOT_FOR_UPDATE = new UninstallMode("Always, but not for update", InstallerConstants.UNINSTALL_NOT_FOR_UPDATE);

    private String description;
    private int intValue;

    private UninstallMode(String description, int value) {
        this.description = description;
        this.intValue = value;
        UninstallMode.intValueToUninstallMode.put(new Integer(intValue), this);
    }

    public String toString() {
        return description;
    }

    /**
     * Convert to an int value.
     * @return the int value
     */
    public int getIntValue() {
        return intValue;
    }

    /**
     * Convert int value to constant.
     * @param uninstall the int value
     * @return the constant
     */
    public static UninstallMode getFromIntValue(int uninstall) {
        return (UninstallMode)UninstallMode.intValueToUninstallMode.get(new Integer(uninstall));
    }
}
