package com.install4j.api.events;

/**
 * Enumeration class that represents all different event types for instances of {@link InstallerEvent}.
 * Installer events cover everything that is written to the installer log file as well
 * as application life-cycle events.
 * @author ej-technologies GmbH
 */
public class EventType {

    /**
     * Fired before an action is executed
     */
    public static final EventType BEFORE_EXECUTE_ACTION = new EventType("Before execute action", "Execute action");

    /**
     * Fired after an action has been executed
     */
    public static final EventType AFTER_EXECUTE_ACTION = new EventType("After execute action", "Execute action");

    /**
     * Fired before a rollback is performed
     */
    public static final EventType BEFORE_ROLLBACK_ACTION = new EventType("Before rollback action", "Rollback action");

    /**
     * Fired after a rollback is performed
     */
    public static final EventType AFTER_ROLLBACK_ACTION = new EventType("After rollback action", "Rollback action");

    /**
     * Fired before a file is installed. The event object will be of type {@link InstallerFileEvent}.
     */
    public static final EventType BEFORE_INSTALL_FILE = new EventType("Before install file", "Install file");

    /**
     * Fired after a file has been installed. The event object will be of type {@link InstallerFileEvent}.
     */
    public static final EventType AFTER_INSTALL_FILE = new EventType("After install file", "Install file");

    /**
     * Fired before a screen is shown.
     */
    public static final EventType SHOW_SCREEN = new EventType("Show screen");

    /**
     * Fired when the installer or uninstaller quits but before any cleanup has been performed.
     */
    public static final EventType FINISHING = new EventType("Finishing");

    /**
     * Fired just before the installer or uninstaller quits.
     */
    public static final EventType FINISHED = new EventType("Finished");

    /**
     * Fired when the installer or uninstaller is cancelled but before any cleanup has been performed.
     */
    public static final EventType CANCELLING = new EventType("Cancelling");

    /**
     * Fired just before the installer or uninstaller quits after having been cancelled.
     */
    public static final EventType CANCELED = new EventType("Canceled");

    /**
     * Fired after a variable has been set.
     * @see com.install4j.api.context.Context#setVariable(String, Object)
     */
    public static final EventType VARIABLE_CHANGED = new EventType("Variable changed");

    /**
     * Fired before the file installation is started by the "Install files" action. You can get the total
     * size and the total file count from the event object
     *
     */
    public static final EventType FILE_INSTALLATION_STARTED = new EventType("File installation started");

    private String eventVerbose;
    private String operationVerbose;

    private EventType(String eventVerbose, String operationVerbose) {
        this.operationVerbose = operationVerbose;
        this.eventVerbose = eventVerbose;
    }

    private EventType(String eventVerbose) {
        this.operationVerbose = eventVerbose;
        this.eventVerbose = eventVerbose;
    }

    /**
     * Get a verbose message that describes the operation that is connected with this event.
     * @return the message
     */
    public String getOperationVerbose() {
        return operationVerbose;
    }

    public String toString() {
        return eventVerbose;
    }
}
