package com.install4j.api.formcomponents;

import com.install4j.api.context.UserCanceledException;
import com.install4j.api.screens.Console;
import com.install4j.api.screens.Screen;

import javax.swing.event.ChangeListener;

/**
 * The form environment gives access to form components in a form panel as well as to other design time
 * or runtime services related to form components.
 * <p>It provides access to the <tt>ComponentTuple</tt> of a form component
 * as well as to the IDs that the framework assigns to form components.
 * Furthermore, the form environment contains utility methods that can be used at design time.
 * This class is passed to the containing screen as well as to every form component.
 *
 * @author ej-technologies GmbH
 * @see com.install4j.api.screens.FormPanelContainer#setFormPanel(javax.swing.JPanel, FormEnvironment)
 * @see FormComponent#setFormEnvironment(FormEnvironment)
 */
public interface FormEnvironment {

    /**
     * Returns all form components that are contained in the same form.
     * Note that the <tt>setFormEnvironment</tt> is called twice at runtime
     * and that only the second invocation passes a <tt>FormEnvironment</tt> that
     * returns all form components when calling this method.
     * @return an array of all form components or the empty array if the form is not fully initialized
     * @see FormComponent#setFormEnvironment(FormEnvironment)
     */
    FormComponent[] getFormComponents();

    /**
     * Get the ID that the framework associates with a form component.
     * When form components refer to other form components and wish to transfer
     * those associations between design time and runtime, they have to store a
     * IDs rather than the form components themselves. The translation between IDs
     * and form components is provided by the form environment.
     * <p>
     * The ID can be transformed back to the form component with the
     * <tt>getFormComponentById</tt> method.
     * @param formComponent the form component whose ID should be returned
     * @return the ID
     * @see #getFormComponentById(String)
     */
    String getId(FormComponent formComponent);

    /**
     * Get the form component of a certain ID. See the
     * <tt>getId</tt> method for more details.
     * @param id the ID of the form component
     * @return the form component or <tt>null</tt> if no such ID can be found.
     * @see #getId(FormComponent)
     */
    FormComponent getFormComponentById(String id);

    /**
     * Get the <tt>ComponentTuple</tt> associated with a form component.
     * @param formComponent the form component whose <tt>ComponentTuple</tt> should be returned
     * @return the <tt>ComponentTuple</tt>
     */
    ComponentTuple getComponentTuple(FormComponent formComponent);

    /**
     * Handle console mode for all form components in a form. This method
     * has to be called at some point in the <tt>handleConsole</tt> method
     * of the containing screen, otherwise the <tt>handleConsole</tt> method of the
     * <tt>FormComponent</tt> will not be called.
     * @param console the <tt>Console</tt> object as passed to the <tt>handleConsole</tt> method of the containing screen.
     * @return whether the installer should continue or not. This value should be passed on in the
     * <tt>handleConsole</tt> method of the containing screen which is the only place where this method should be called.
     * @throws UserCanceledException if the user cancels a question or notice. This exception can be passed on to the framework.
     * @see FormComponent#handleConsole(com.install4j.api.screens.Console)
     * @see com.install4j.api.screens.Screen#handleConsole(com.install4j.api.screens.Console)
     */
    boolean handleConsole(Console console) throws UserCanceledException;

    /**
     * Returns whether the form component has been instantiated at design time.
     * At design time, the context passed to a form component is <tt>null</tt>.
     * @return <tt>true</tt> or <tt>false</tt>.
     */
    boolean isDesignTime();

    /**
     * If your form component would like to be notified when the list of form components changes at design time,
     * you can add a change listener here. Only a weak reference to the listener is held,
     * so the garbage collection of your form component is not influenced.
     * <p>This method has no effect at runtime.
     * @param changeListener the listener
     */
    void addDesignTimeChangeListener(ChangeListener changeListener);

    /**
     * Remove a change listener added with <tt>addDesignTimeChangeListener</tt>.
     * <p>This method has no effect at runtime.
     * @param changeListener the listener
     */
    void removeDesignTimeChangeListener(ChangeListener changeListener);

    /**
     * At design time, the install4j GUI allows the user to name instances of form components in a form.
     * You can retrieve that name with this method.
     * @param formComponent the form component whose name should be returned.
     * @return name the design time name or <tt>null</tt> if called at runtime
     */
    String getDesignTimeName(FormComponent formComponent);

    /**
     * Get the form screen associated with this form environment.
     * @return the screen
     */
    Screen getScreen();

    /**
     * All form components whose "Reset initialization on previous" property is selected,
     * will be initialized again <b>when the screen is shown the next time</b>.
     * Usually, initialization is only reset when the user presses the "Back button" on the
     * form screen. If you implement a loop and want to reset the initialization, you
     * have to call this method. The following code retrieves a form screen by ID and resets its form components:
     * <pre>
     *
     * String screenId = ...
     * // must be a form screen for the cast to work
     * FormPanelContainer formPanelContainer = (FormPanelContainer)context.getScreenById(id);
     * formPanelContainer.getFormEnvironment().resetFormComponents()
     *
     * </pre>
     */
    void resetFormComponents();

    /**
     * All form components will be validated and their variables will be saved.
     * This is what happens automatically when the user clicks "Next" just before the validation expression
     * of the screen is evaluated. If you need save the form componments to variables whil the screen is being shown,
     * you can use this method. For example, a button form component might require some variables of other form components
     * to be set to the currently entered values.
     *
     * @return if the validation was sucessful. If <tt>false</tt>, only form components up to the offending form component
     * are saved.
     */
    boolean saveFormComponents();
}
