package com.install4j.api.windows;

import com.install4j.runtime.installer.platform.win32.Registry;

/**
 * Collection of static methods to access the Microsoft Windows registry.
 * @author ej-technologies GmbH
 */
public class WinRegistry {

    /**
     * Checks if a registry key exists.
     * @param root one of the <tt>RegistryRoot.*</tt> constants.
     * @param keyName the name of the key without a leading backslash.
     * @return <tt>true</tt> if the key exists.
     */
    public static boolean keyExists(RegistryRoot root, String keyName) {
        return Registry.keyExists(root.getType(), keyName);
    }

    /**
     * Creates a registry key.
     * @param root one of the <tt>RegistryRoot.*</tt> constants.
     * @param keyName the name of the key without a leading backslash.
     * @return <tt>true</tt> if successful.
     */
    public static boolean createKey(RegistryRoot root, String keyName) {
        return Registry.createKey(root.getType(), keyName);
    }

    /**
     * Retrieves a registry value. The class of the returned object is determined
     * by the value's type.
     * @param root one of the <tt>RegistryRoot.*</tt> constants.
     * @param keyName the name of the key without a leading backslash.
     * @param valueName the name of the registry value.
     * @return the requested value. Returns <tt>null</tt>, if the value doesn't exist. Otherwise
     * it is an instance of one of the following classes: <tt>String, Integer, String[], byte[], WinRegistry.ExpandString</tt>.
     */
    public static Object getValue(RegistryRoot root, String keyName, String valueName) {
        return Registry.getValue(root.getType(), keyName, valueName);
    }

    /**
     * Sets a registry value. The type is determined by the class of the value.
     * @param root one of the <tt>RegistryRoot.*</tt> constants
     * @param keyName the name of the key without a leading backslash.
     * @param valueName the name of the registry value.
     * @param value an instance of one of the following classes: <tt>String, Integer,
     * String[], byte[], WinRegistry.ExpandString</tt>.
     * @return <tt>true</tt> if successful.
     */
    public static boolean setValue(RegistryRoot root, String keyName, String valueName, Object value) {
        return Registry.setValue(root.getType(), keyName, valueName, value);
    }

    /**
     * Deletes a registry value.
     * @param root one of the <tt>RegistryRoot.*</tt> constants.
     * @param keyName the name of the key without a leading backslash.
     * @param valueName the name of the registry value.
     */
    public static void deleteValue(RegistryRoot root, String keyName, String valueName) {
        Registry.deleteValue(root.getType(), keyName, valueName);
    }

    /**
     * Deletes a registry key.
     * @param root one of the <tt>RegistryRoot.*</tt> constants.
     * @param keyName the name of the key without a leading backslash.
     * @param onlyIfEmpty if <tt>true</tt>, the method will delete the key only if the key contains no
     * values and no subkey.
     */
    public static void deleteKey(RegistryRoot root, String keyName, boolean onlyIfEmpty) {
        Registry.deleteKey(root.getType(), keyName, onlyIfEmpty);
    }

    /**
     * Retrieves the sub keys of a registry key.
     * @param root one of the <tt>RegistryRoot.*</tt> constants.
     * @param keyName the name of the key without a leading backslash.
     * @return an array with the sub keys' names. Returns an empty array if there are no sub keys and
     * <tt>null</tt> if the key doesn't exist or if an error occurs.
     */
    public static String[] getSubKeyNames(RegistryRoot root, String keyName) {
        return Registry.enumSubKeys(root.getType(), keyName);
    }

    /**
     * Retrieves the exiting value names of a registry key.
     * @param root one of the <tt>RegistryRoot.*</tt> constants.
     * @param keyName the name of the key without a leading backslash.
     * @return an array with the values' names. Returns an empty array if there are no values and
     * <tt>null</tt> if the key doesn't exist or if an error occurs.
     */
    public static String[] getValueNames(RegistryRoot root, String keyName) {
        return Registry.enumValues(root.getType(), keyName);
    }

    /**
     * Save a sub-tree of the registry to a file.
     * The sub-tree can be restored with <tt>restoreKey</tt>.
     * @param root one of the <tt>RegistryRoot.*</tt> constants.
     * @param keyName the name of the key without a leading backslash. This is the root of the saved sub-tree.
     * @param fileName the file to which the sub-tree should be saved.
     * @return whether the operation was sucessful or not.
     * @see #restoreKey(RegistryRoot, String, String)
     */
    public static boolean saveKey(RegistryRoot root, String keyName, String fileName) {
        return Registry.saveKey(root.getType(), keyName, fileName);
    }

    /**
     * Restore a sub-tree of the registry from a file.
     * This method can restore files saved with the <tt>saveKey</tt> method.
     * @param root one of the <tt>RegistryRoot.*</tt> constants.
     * @param keyName the name of the key without a leading backslash. This is the key to which the root of the saved sub-tree
     * will be restored.
     * @param fileName the file from which the saved sub-tree should be read.
     * @return whether the operation was successful or not.
     * @see #saveKey(RegistryRoot, String, String)
     */
    public static boolean restoreKey(RegistryRoot root, String keyName, String fileName) {
        return Registry.restoreKey(root.getType(), keyName, fileName);
    }

    /**
     * Class to represent Strings with type REG_EXPAND_SZ.
     */
    public static class ExpandString {
        private String value;

        /**
         * Constructor.
         * @param value the string value
         */
        public ExpandString(String value) {
            this.value = value;
        }

        /**
         * Get the string value.
         * @return the string value
         */
        public String getValue() {
            return value;
        }

        public String toString() {
            return value;
        }
    }


    private WinRegistry() {
    }
}
